@@ -160,6 +160,11 @@ var knownGlobals = [][]string{
 	{"Reflect", "set"},
 	{"Reflect", "setPrototypeOf"},
 
+	// JSON: Static Methods
+	// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/JSON#static_methods
+	{"JSON", "parse"},
+	{"JSON", "stringify"},
+
 	// Other globals present in both the browser and node (except "eval" because
 	// it has special behavior)
 	{"AbortController"},
