@@ -6,6 +6,7 @@ import (
 	"unicode/utf8"
 
 	"github.com/evanw/esbuild/internal/ast"
+	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
 	"github.com/evanw/esbuild/internal/css_ast"
 	"github.com/evanw/esbuild/internal/css_lexer"
@@ -19,7 +20,9 @@ type printer struct {
 	options                Options
 	importRecords          []ast.ImportRecord
 	css                    []byte
-	extractedLegalComments map[string]bool
+	hasLegalComment        map[string]struct{}
+	extractedLegalComments []string
+	jsonMetadataImports    []string
 	builder                sourcemap.ChunkBuilder
 }
 
@@ -32,17 +35,24 @@ type Options struct {
 	// us do binary search on to figure out what line a given AST node came from
 	LineOffsetTables []sourcemap.LineOffsetTable
 
-	MinifyWhitespace  bool
-	ASCIIOnly         bool
-	SourceMap         config.SourceMap
-	AddSourceMappings bool
-	LegalComments     config.LegalComments
+	UnsupportedFeatures compat.CSSFeature
+	MinifyWhitespace    bool
+	ASCIIOnly           bool
+	SourceMap           config.SourceMap
+	AddSourceMappings   bool
+	LegalComments       config.LegalComments
+	NeedsMetafile       bool
 }
 
 type PrintResult struct {
 	CSS                    []byte
-	ExtractedLegalComments map[string]bool
-	SourceMapChunk         sourcemap.Chunk
+	ExtractedLegalComments []string
+	JSONMetadataImports    []string
+
+	// This source map chunk just contains the VLQ-encoded offsets for the "CSS"
+	// field above. It's not a full source map. The bundler will be joining many
+	// source map chunks together to form the final source map.
+	SourceMapChunk sourcemap.Chunk
 }
 
 func Print(tree css_ast.AST, options Options) PrintResult {
@@ -57,6 +67,7 @@ func Print(tree css_ast.AST, options Options) PrintResult {
 	result := PrintResult{
 		CSS:                    p.css,
 		ExtractedLegalComments: p.extractedLegalComments,
+		JSONMetadataImports:    p.jsonMetadataImports,
 	}
 	if options.SourceMap != config.SourceMapNone {
 		// This is expensive. Only do this if it's necessary. For example, skipping
@@ -67,6 +78,20 @@ func Print(tree css_ast.AST, options Options) PrintResult {
 	return result
 }
 
+func (p *printer) recordImportPathForMetafile(importRecordIndex uint32) {
+	if p.options.NeedsMetafile {
+		record := p.importRecords[importRecordIndex]
+		external := ""
+		if (record.Flags & ast.ShouldNotBeExternalInMetafile) == 0 {
+			external = ",\n          \"external\": true"
+		}
+		p.jsonMetadataImports = append(p.jsonMetadataImports, fmt.Sprintf("\n        {\n          \"path\": %s,\n          \"kind\": %s%s\n        }",
+			helpers.QuoteForJSON(record.Path.Text, p.options.ASCIIOnly),
+			helpers.QuoteForJSON(record.Kind.StringForMetafile(), p.options.ASCIIOnly),
+			external))
+	}
+}
+
 func (p *printer) printRule(rule css_ast.Rule, indent int32, omitTrailingSemicolon bool) {
 	if r, ok := rule.Data.(*css_ast.RComment); ok {
 		switch p.options.LegalComments {
@@ -76,10 +101,15 @@ func (p *printer) printRule(rule css_ast.Rule, indent int32, omitTrailingSemicol
 		case config.LegalCommentsEndOfFile,
 			config.LegalCommentsLinkedWithComment,
 			config.LegalCommentsExternalWithoutComment:
-			if p.extractedLegalComments == nil {
-				p.extractedLegalComments = make(map[string]bool)
+
+			// Don't record the same legal comment more than once per file
+			if p.hasLegalComment == nil {
+				p.hasLegalComment = make(map[string]struct{})
+			} else if _, ok := p.hasLegalComment[r.Text]; ok {
+				return
 			}
-			p.extractedLegalComments[r.Text] = true
+			p.hasLegalComment[r.Text] = struct{}{}
+			p.extractedLegalComments = append(p.extractedLegalComments, r.Text)
 			return
 		}
 	}
@@ -108,6 +138,7 @@ func (p *printer) printRule(rule css_ast.Rule, indent int32, omitTrailingSemicol
 			p.print("@import ")
 		}
 		p.printQuoted(p.importRecords[r.ImportRecordIndex].Path.Text)
+		p.recordImportPathForMetafile(r.ImportRecordIndex)
 		p.printTokens(r.ImportConditions, printTokensOpts{})
 		p.print(";")
 
@@ -268,7 +299,9 @@ func (p *printer) printRule(rule css_ast.Rule, indent int32, omitTrailingSemicol
 
 func (p *printer) printIndentedComment(indent int32, text string) {
 	// Avoid generating a comment containing the character sequence "</style"
-	text = helpers.EscapeClosingTag(text, "/style")
+	if !p.options.UnsupportedFeatures.Has(compat.InlineStyle) {
+		text = helpers.EscapeClosingTag(text, "/style")
+	}
 
 	// Re-indent multi-line comments
 	for {
@@ -580,7 +613,7 @@ func (p *printer) printQuotedWithQuote(text string, quote byte) {
 
 		case '/':
 			// Avoid generating the sequence "</style" in CSS code
-			if i >= 1 && text[i-1] == '<' && i+6 <= len(text) && strings.EqualFold(text[i+1:i+6], "style") {
+			if !p.options.UnsupportedFeatures.Has(compat.InlineStyle) && i >= 1 && text[i-1] == '<' && i+6 <= len(text) && strings.EqualFold(text[i+1:i+6], "style") {
 				escape = escapeBackslash
 			}
 
@@ -779,6 +812,7 @@ func (p *printer) printTokens(tokens []css_ast.Token, opts printTokensOpts) bool
 			p.print("url(")
 			p.printQuotedWithQuote(text, bestQuoteCharForString(text, true))
 			p.print(")")
+			p.recordImportPathForMetafile(t.ImportRecordIndex)
 
 		default:
 			p.print(t.Text)
