@@ -4,13 +4,15 @@ import "strings"
 
 var builtinTypesLower = map[string]string{
 	// Text
-	".css":  "text/css; charset=utf-8",
-	".htm":  "text/html; charset=utf-8",
-	".html": "text/html; charset=utf-8",
-	".js":   "text/javascript; charset=utf-8",
-	".json": "application/json",
-	".mjs":  "text/javascript; charset=utf-8",
-	".xml":  "text/xml; charset=utf-8",
+	".css":      "text/css; charset=utf-8",
+	".htm":      "text/html; charset=utf-8",
+	".html":     "text/html; charset=utf-8",
+	".js":       "text/javascript; charset=utf-8",
+	".json":     "application/json",
+	".markdown": "text/markdown; charset=utf-8",
+	".md":       "text/markdown; charset=utf-8",
+	".mjs":      "text/javascript; charset=utf-8",
+	".xml":      "text/xml; charset=utf-8",
 
 	// Images
 	".avif": "image/avif",
