@@ -52,14 +52,41 @@ func TestLowerArrowFunction(t *testing.T) {
 }
 
 func TestLowerNullishCoalescing(t *testing.T) {
-	expectParseError(t, "a ?? b && c", "<stdin>: ERROR: Unexpected \"&&\"\n")
-	expectParseError(t, "a ?? b || c", "<stdin>: ERROR: Unexpected \"||\"\n")
-	expectParseError(t, "a ?? b && c || d", "<stdin>: ERROR: Unexpected \"&&\"\n")
-	expectParseError(t, "a ?? b || c && d", "<stdin>: ERROR: Unexpected \"||\"\n")
-	expectParseError(t, "a && b ?? c", "<stdin>: ERROR: Unexpected \"??\"\n")
-	expectParseError(t, "a || b ?? c", "<stdin>: ERROR: Unexpected \"??\"\n")
-	expectParseError(t, "a && b || c ?? c", "<stdin>: ERROR: Unexpected \"??\"\n")
-	expectParseError(t, "a || b && c ?? d", "<stdin>: ERROR: Unexpected \"??\"\n")
+	expectParseError(t, "a ?? b && c",
+		"<stdin>: ERROR: Cannot use \"&&\" with \"??\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x ?? y && z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x ?? y) && z\" and \"x ?? (y && z)\" by adding parentheses.\n")
+	expectParseError(t, "a ?? b || c",
+		"<stdin>: ERROR: Cannot use \"||\" with \"??\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x ?? y || z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x ?? y) || z\" and \"x ?? (y || z)\" by adding parentheses.\n")
+	expectParseError(t, "a ?? b && c || d",
+		"<stdin>: ERROR: Cannot use \"&&\" with \"??\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x ?? y && z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x ?? y) && z\" and \"x ?? (y && z)\" by adding parentheses.\n"+
+			"<stdin>: ERROR: Cannot use \"||\" with \"??\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x ?? y || z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x ?? y) || z\" and \"x ?? (y || z)\" by adding parentheses.\n")
+	expectParseError(t, "a ?? b || c && d",
+		"<stdin>: ERROR: Cannot use \"||\" with \"??\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x ?? y || z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x ?? y) || z\" and \"x ?? (y || z)\" by adding parentheses.\n")
+	expectParseError(t, "a && b ?? c",
+		"<stdin>: ERROR: Cannot use \"??\" with \"&&\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x && y ?? z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x && y) ?? z\" and \"x && (y ?? z)\" by adding parentheses.\n")
+	expectParseError(t, "a || b ?? c",
+		"<stdin>: ERROR: Cannot use \"??\" with \"||\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x || y ?? z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x || y) ?? z\" and \"x || (y ?? z)\" by adding parentheses.\n")
+	expectParseError(t, "a && b || c ?? c",
+		"<stdin>: ERROR: Cannot use \"??\" with \"||\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x || y ?? z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x || y) ?? z\" and \"x || (y ?? z)\" by adding parentheses.\n")
+	expectParseError(t, "a || b && c ?? d",
+		"<stdin>: ERROR: Cannot use \"??\" with \"||\" without parentheses\n"+
+			"NOTE: Expressions of the form \"x || y ?? z\" are not allowed in JavaScript. "+
+			"You must disambiguate between \"(x || y) ?? z\" and \"x || (y ?? z)\" by adding parentheses.\n")
 	expectPrinted(t, "a ?? b, b && c", "a ?? b, b && c;\n")
 	expectPrinted(t, "a ?? b, b || c", "a ?? b, b || c;\n")
 	expectPrinted(t, "a && b, b ?? c", "a && b, b ?? c;\n")
@@ -651,6 +678,10 @@ func TestLowerOptionalChain(t *testing.T) {
   }
 }
 `)
+
+	expectPrintedTarget(t, 2020, "(x?.y)``", "(x?.y)``;\n")
+	expectPrintedTarget(t, 2019, "(x?.y)``", "var _a;\n(x == null ? void 0 : x.y).call(x, _a || (_a = __template([\"\"])));\n")
+	expectPrintedTarget(t, 5, "(x?.y)``", "var _a;\n(x == null ? void 0 : x.y).call(x, _a || (_a = __template([\"\"])));\n")
 }
 
 func TestLowerOptionalCatchBinding(t *testing.T) {
