@@ -1,4 +1,4 @@
-import * as types from "../shared/types"
+import type * as types from "../shared/types"
 import * as common from "../shared/common"
 import * as ourselves from "./wasm"
 
@@ -8,15 +8,15 @@ declare let WEB_WORKER_FUNCTION: (postMessage: (data: Uint8Array) => void) => (e
 
 export let version = ESBUILD_VERSION
 
-export let build: typeof types.build = (options: types.BuildOptions): Promise<any> =>
+export let build: typeof types.build = (options: types.BuildOptions) =>
   ensureServiceIsRunning().then(service =>
     service.build(options))
 
-export const serve: typeof types.serve = () => {
-  throw new Error(`The "serve" API does not work in Deno via WebAssembly`)
-}
+export const context: typeof types.context = (options: types.BuildOptions) =>
+  ensureServiceIsRunning().then(service =>
+    service.context(options))
 
-export const transform: typeof types.transform = (input, options) =>
+export const transform: typeof types.transform = (input: string | Uint8Array, options?: types.TransformOptions) =>
   ensureServiceIsRunning().then(service =>
     service.transform(input, options))
 
@@ -50,6 +50,7 @@ export const stop = () => {
 
 interface Service {
   build: typeof types.build
+  context: typeof types.context
   transform: typeof types.transform
   formatMessages: typeof types.formatMessages
   analyzeMetafile: typeof types.analyzeMetafile
@@ -59,7 +60,7 @@ let initializePromise: Promise<Service> | undefined;
 let stopService: (() => void) | undefined
 
 let ensureServiceIsRunning = (): Promise<Service> => {
-  return initializePromise || startRunningService('', undefined, true)
+  return initializePromise || startRunningService('esbuild.wasm', undefined, true)
 }
 
 export const initialize: typeof types.initialize = async (options) => {
@@ -68,7 +69,7 @@ export const initialize: typeof types.initialize = async (options) => {
   let wasmModule = options.wasmModule;
   let useWorker = options.worker !== false;
   if (initializePromise) throw new Error('Cannot call "initialize" more than once');
-  initializePromise = startRunningService(wasmURL || '', wasmModule, useWorker);
+  initializePromise = startRunningService(wasmURL || 'esbuild.wasm', wasmModule, useWorker);
   initializePromise.catch(() => {
     // Let the caller try again if this fails
     initializePromise = void 0;
@@ -119,7 +120,7 @@ const startRunningService = async (wasmURL: string | URL, wasmModule: WebAssembl
       worker.postMessage(bytes)
     },
     isSync: false,
-    isWriteUnavailable: true,
+    hasFS: false,
     esbuild: ourselves,
   })
 
@@ -133,19 +134,30 @@ const startRunningService = async (wasmURL: string | URL, wasmModule: WebAssembl
   }
 
   return {
-    build: (options: types.BuildOptions): Promise<any> =>
+    build: (options: types.BuildOptions) =>
       new Promise<types.BuildResult>((resolve, reject) =>
-        service.buildOrServe({
+        service.buildOrContext({
           callName: 'build',
           refs: null,
-          serveOptions: null,
           options,
           isTTY: false,
           defaultWD: '/',
           callback: (err, res) => err ? reject(err) : resolve(res as types.BuildResult),
         })),
-    transform: (input, options) =>
-      new Promise((resolve, reject) =>
+
+    context: (options: types.BuildOptions) =>
+      new Promise<types.BuildContext>((resolve, reject) =>
+        service.buildOrContext({
+          callName: 'context',
+          refs: null,
+          options,
+          isTTY: false,
+          defaultWD: '/',
+          callback: (err, res) => err ? reject(err) : resolve(res as types.BuildContext),
+        })),
+
+    transform: (input: string | Uint8Array, options?: types.TransformOptions) =>
+      new Promise<types.TransformResult>((resolve, reject) =>
         service.transform({
           callName: 'transform',
           refs: null,
@@ -158,6 +170,7 @@ const startRunningService = async (wasmURL: string | URL, wasmModule: WebAssembl
           },
           callback: (err, res) => err ? reject(err) : resolve(res!),
         })),
+
     formatMessages: (messages, options) =>
       new Promise((resolve, reject) =>
         service.formatMessages({
@@ -167,6 +180,7 @@ const startRunningService = async (wasmURL: string | URL, wasmModule: WebAssembl
           options,
           callback: (err, res) => err ? reject(err) : resolve(res!),
         })),
+
     analyzeMetafile: (metafile, options) =>
       new Promise((resolve, reject) =>
         service.analyzeMetafile({
