@@ -4,7 +4,7 @@
 // additional byte array primitive. You must send a response after receiving a
 // request because the other end is blocking on the response coming back.
 
-import * as types from "./types";
+import type * as types from "./types";
 
 export interface BuildRequest {
   command: 'build';
@@ -15,17 +15,21 @@ export interface BuildRequest {
   stdinContents: Uint8Array | null;
   stdinResolveDir: string | null;
   absWorkingDir: string;
-  incremental: boolean;
   nodePaths: string[];
+  context: boolean;
   plugins?: BuildPlugin[];
-  serve?: ServeRequest;
   mangleCache?: Record<string, string | false>;
 }
 
 export interface ServeRequest {
+  command: 'serve';
+  key: number;
+  onRequest: boolean;
   port?: number;
   host?: string;
   servedir?: string;
+  keyfile?: string;
+  certfile?: string;
 }
 
 export interface ServeResponse {
@@ -33,13 +37,10 @@ export interface ServeResponse {
   host: string;
 }
 
-export interface ServeStopRequest {
-  command: 'serve-stop';
-  key: number;
-}
-
 export interface BuildPlugin {
   name: string;
+  onStart: boolean;
+  onEnd: boolean;
   onResolve: { id: number, filter: string, namespace: string }[];
   onLoad: { id: number, filter: string, namespace: string }[];
 }
@@ -47,14 +48,21 @@ export interface BuildPlugin {
 export interface BuildResponse {
   errors: types.Message[];
   warnings: types.Message[];
-  rebuild: boolean;
-  watch: boolean;
   outputFiles?: BuildOutputFile[];
   metafile?: string;
   mangleCache?: Record<string, string | false>;
   writeToStdout?: Uint8Array;
 }
 
+export interface OnEndRequest extends BuildResponse {
+  command: 'on-end';
+}
+
+export interface OnEndResponse {
+  errors: types.Message[];
+  warnings: types.Message[];
+}
+
 export interface BuildOutputFile {
   path: string;
   contents: Uint8Array;
@@ -69,34 +77,27 @@ export interface RebuildRequest {
   key: number;
 }
 
-export interface RebuildDisposeRequest {
-  command: 'rebuild-dispose';
+export interface RebuildResponse {
+  errors: types.Message[];
+  warnings: types.Message[];
+}
+
+export interface DisposeRequest {
+  command: 'dispose';
   key: number;
 }
 
-export interface WatchStopRequest {
-  command: 'watch-stop';
+export interface WatchRequest {
+  command: 'watch';
   key: number;
 }
 
-export interface OnRequestRequest {
+export interface OnServeRequest {
   command: 'serve-request';
   key: number;
   args: types.ServeOnRequestArgs;
 }
 
-export interface OnWaitRequest {
-  command: 'serve-wait';
-  key: number;
-  error: string | null;
-}
-
-export interface OnWatchRebuildRequest {
-  command: 'watch-rebuild';
-  key: number;
-  args: BuildResponse;
-}
-
 export interface TransformRequest {
   command: 'transform';
   flags: string[];
@@ -115,6 +116,7 @@ export interface TransformResponse {
   map: string;
   mapFS: boolean;
 
+  legalComments?: string;
   mangleCache?: Record<string, string | false>;
 }
 
@@ -394,6 +396,7 @@ class ByteBuffer {
 
 export let encodeUTF8: (text: string) => Uint8Array
 export let decodeUTF8: (bytes: Uint8Array) => string
+let encodeInvariant: string
 
 // For the browser and node 12.x
 if (typeof TextEncoder !== 'undefined' && typeof TextDecoder !== 'undefined') {
@@ -401,37 +404,34 @@ if (typeof TextEncoder !== 'undefined' && typeof TextDecoder !== 'undefined') {
   let decoder = new TextDecoder();
   encodeUTF8 = text => encoder.encode(text);
   decodeUTF8 = bytes => decoder.decode(bytes);
+  encodeInvariant = 'new TextEncoder().encode("")'
 }
 
 // For node 10.x
 else if (typeof Buffer !== 'undefined') {
-  encodeUTF8 = text => {
-    let buffer: Uint8Array = Buffer.from(text);
-
-    // The test framework called "Jest" breaks node's Buffer API. Normally
-    // instances of Buffer are also instances of Uint8Array, but not when
-    // esbuild is run inside of whatever weird environment Jest uses. More
-    // info: https://github.com/facebook/jest/issues/4422.
-    if (!(buffer instanceof Uint8Array)) {
-      // Construct a new Uint8Array with the contents of the buffer to force
-      // it to be a Uint8Array instance. This is wasteful since it's slower
-      // than just using the Buffer, but this should only happen when esbuild
-      // is run inside of Jest.
-      buffer = new Uint8Array(buffer);
-    }
-
-    return buffer;
-  };
+  encodeUTF8 = text => Buffer.from(text);
   decodeUTF8 = bytes => {
     let { buffer, byteOffset, byteLength } = bytes;
     return Buffer.from(buffer, byteOffset, byteLength).toString();
   }
+  encodeInvariant = 'Buffer.from("")'
 }
 
 else {
   throw new Error('No UTF-8 codec found');
 }
 
+// Throw an error early if this isn't true. The test framework called "Jest"
+// has some bugs regarding this edge case, and letting esbuild proceed further
+// leads to confusing errors that make it seem like esbuild itself has a bug.
+if (!(encodeUTF8('') instanceof Uint8Array))
+  throw new Error(`Invariant violation: "${encodeInvariant} instanceof Uint8Array" is incorrectly false
+
+This indicates that your JavaScript environment is broken. You cannot use
+esbuild in this environment because esbuild relies on this invariant. This
+is not a problem with esbuild. You need to fix your environment instead.
+`)
+
 export function readUInt32LE(buffer: Uint8Array, offset: number): number {
   return buffer[offset++] |
     (buffer[offset++] << 8) |
