@@ -1,6 +1,6 @@
 export type Platform = 'browser' | 'node' | 'neutral';
 export type Format = 'iife' | 'cjs' | 'esm';
-export type Loader = 'js' | 'jsx' | 'ts' | 'tsx' | 'css' | 'json' | 'text' | 'base64' | 'file' | 'dataurl' | 'binary' | 'copy' | 'default';
+export type Loader = 'base64' | 'binary' | 'copy' | 'css' | 'dataurl' | 'default' | 'empty' | 'file' | 'js' | 'json' | 'jsx' | 'text' | 'ts' | 'tsx';
 export type LogLevel = 'verbose' | 'debug' | 'info' | 'warning' | 'error' | 'silent';
 export type Charset = 'ascii' | 'utf8';
 export type Drop = 'console' | 'debugger';
@@ -98,6 +98,8 @@ export interface BuildOptions extends CommonOptions {
   outbase?: string;
   /** Documentation: https://esbuild.github.io/api/#external */
   external?: string[];
+  /** Documentation: https://esbuild.github.io/api/#packages */
+  packages?: 'external';
   /** Documentation: https://esbuild.github.io/api/#alias */
   alias?: Record<string, string>;
   /** Documentation: https://esbuild.github.io/api/#loader */
@@ -130,8 +132,6 @@ export interface BuildOptions extends CommonOptions {
   banner?: { [type: string]: string };
   /** Documentation: https://esbuild.github.io/api/#footer */
   footer?: { [type: string]: string };
-  /** Documentation: https://esbuild.github.io/api/#incremental */
-  incremental?: boolean;
   /** Documentation: https://esbuild.github.io/api/#entry-points */
   entryPoints?: string[] | Record<string, string>;
   /** Documentation: https://esbuild.github.io/api/#stdin */
@@ -142,12 +142,6 @@ export interface BuildOptions extends CommonOptions {
   absWorkingDir?: string;
   /** Documentation: https://esbuild.github.io/api/#node-paths */
   nodePaths?: string[]; // The "NODE_PATH" variable from Node.js
-  /** Documentation: https://esbuild.github.io/api/#watch */
-  watch?: boolean | WatchMode;
-}
-
-export interface WatchMode {
-  onRebuild?: (error: BuildFailure | null, result: BuildResult | null) => void;
 }
 
 export interface StdinOptions {
@@ -193,32 +187,19 @@ export interface OutputFile {
   path: string;
   /** "text" as bytes */
   contents: Uint8Array;
-  /** "contents" as text */
-  text: string;
+  /** "contents" as text (changes automatically with "contents") */
+  readonly text: string;
 }
 
-export interface BuildInvalidate {
-  (): Promise<BuildIncremental>;
-  dispose(): void;
-}
-
-export interface BuildIncremental extends BuildResult {
-  rebuild: BuildInvalidate;
-}
-
-export interface BuildResult {
+export interface BuildResult<SpecificOptions extends BuildOptions = BuildOptions> {
   errors: Message[];
   warnings: Message[];
   /** Only when "write: false" */
-  outputFiles?: OutputFile[];
-  /** Only when "incremental: true" */
-  rebuild?: BuildInvalidate;
-  /** Only when "watch: true" */
-  stop?: () => void;
+  outputFiles: OutputFile[] | (SpecificOptions['write'] extends false ? never : undefined);
   /** Only when "metafile: true" */
-  metafile?: Metafile;
+  metafile: Metafile | (SpecificOptions['metafile'] extends true ? never : undefined);
   /** Only when "mangleCache" is present */
-  mangleCache?: Record<string, string | false>;
+  mangleCache: Record<string, string | false> | (SpecificOptions['mangleCache'] extends Object ? never : undefined);
 }
 
 export interface BuildFailure extends Error {
@@ -231,6 +212,8 @@ export interface ServeOptions {
   port?: number;
   host?: string;
   servedir?: string;
+  keyfile?: string
+  certfile?: string
   onRequest?: (args: ServeOnRequestArgs) => void;
 }
 
@@ -247,8 +230,6 @@ export interface ServeOnRequestArgs {
 export interface ServeResult {
   port: number;
   host: string;
-  wait: Promise<void>;
-  stop: () => void;
 }
 
 export interface TransformOptions extends CommonOptions {
@@ -272,12 +253,14 @@ export interface TransformOptions extends CommonOptions {
   footer?: string;
 }
 
-export interface TransformResult {
+export interface TransformResult<SpecificOptions extends TransformOptions = TransformOptions> {
   code: string;
   map: string;
   warnings: Message[];
   /** Only when "mangleCache" is present */
-  mangleCache?: Record<string, string | false>;
+  mangleCache: Record<string, string | false> | (SpecificOptions['mangleCache'] extends Object ? never : undefined);
+  /** Only when "legalComments" is "external" */
+  legalComments: string | (SpecificOptions['legalComments'] extends 'external' ? never : undefined);
 }
 
 export interface TransformFailure extends Error {
@@ -297,7 +280,7 @@ export interface PluginBuild {
   onStart(callback: () =>
     (OnStartResult | null | void | Promise<OnStartResult | null | void>)): void;
   onEnd(callback: (result: BuildResult) =>
-    (void | Promise<void>)): void;
+    (OnEndResult | null | void | Promise<OnEndResult | null | void>)): void;
   onResolve(options: OnResolveOptions, callback: (args: OnResolveArgs) =>
     (OnResolveResult | null | undefined | Promise<OnResolveResult | null | undefined>)): void;
   onLoad(options: OnLoadOptions, callback: (args: OnLoadArgs) =>
@@ -305,7 +288,7 @@ export interface PluginBuild {
 
   // This is a full copy of the esbuild library in case you need it
   esbuild: {
-    serve: typeof serve,
+    context: typeof context,
     build: typeof build,
     buildSync: typeof buildSync,
     transform: typeof transform,
@@ -345,6 +328,11 @@ export interface OnStartResult {
   warnings?: PartialMessage[];
 }
 
+export interface OnEndResult {
+  errors?: PartialMessage[];
+  warnings?: PartialMessage[];
+}
+
 export interface OnResolveOptions {
   filter: RegExp;
   namespace?: string;
@@ -437,7 +425,10 @@ export interface Metafile {
       imports: {
         path: string
         kind: ImportKind
+        external?: boolean
+        original?: string
       }[]
+      format?: 'cjs' | 'esm'
     }
   }
   outputs: {
@@ -450,7 +441,8 @@ export interface Metafile {
       }
       imports: {
         path: string
-        kind: ImportKind
+        kind: ImportKind | 'file-loader'
+        external?: boolean
       }[]
       exports: string[]
       entryPoint?: string
@@ -470,6 +462,16 @@ export interface AnalyzeMetafileOptions {
   verbose?: boolean;
 }
 
+export interface WatchOptions {
+}
+
+export interface BuildContext<SpecificOptions extends BuildOptions = BuildOptions> {
+  rebuild(): Promise<BuildResult<SpecificOptions>>
+  watch(options?: WatchOptions): void
+  serve(options?: ServeOptions): Promise<ServeResult>
+  dispose(): Promise<void>
+}
+
 /**
  * This function invokes the "esbuild" command-line tool for you. It returns a
  * promise that either resolves with a "BuildResult" object or rejects with a
@@ -480,22 +482,20 @@ export interface AnalyzeMetafileOptions {
  *
  * Documentation: https://esbuild.github.io/api/#build-api
  */
-export declare function build(options: BuildOptions & { write: false }): Promise<BuildResult & { outputFiles: OutputFile[] }>;
-export declare function build(options: BuildOptions & { incremental: true, metafile: true }): Promise<BuildIncremental & { metafile: Metafile }>;
-export declare function build(options: BuildOptions & { incremental: true }): Promise<BuildIncremental>;
-export declare function build(options: BuildOptions & { metafile: true }): Promise<BuildResult & { metafile: Metafile }>;
+export declare function build<SpecificOptions extends BuildOptions>(options: SpecificOptions): Promise<BuildResult<SpecificOptions>>;
 export declare function build(options: BuildOptions): Promise<BuildResult>;
 
 /**
- * This function is similar to "build" but it serves the resulting files over
- * HTTP on a localhost address with the specified port.
+ * This is the advanced long-running form of "build" that supports additional
+ * features such as watch mode and a local development server.
  *
  * - Works in node: yes
  * - Works in browser: no
  *
- * Documentation: https://esbuild.github.io/api/#serve
+ * Documentation: https://esbuild.github.io/api/#context-api
  */
-export declare function serve(serveOptions: ServeOptions, buildOptions: BuildOptions): Promise<ServeResult>;
+export declare function context<T extends BuildOptions>(options: T): Promise<BuildContext<T>>;
+export declare function context(options: BuildOptions): Promise<BuildContext>;
 
 /**
  * This function transforms a single JavaScript file. It can be used to minify
@@ -508,6 +508,7 @@ export declare function serve(serveOptions: ServeOptions, buildOptions: BuildOpt
  *
  * Documentation: https://esbuild.github.io/api/#transform-api
  */
+export declare function transform<SpecificOptions extends TransformOptions>(input: string | Uint8Array, options?: SpecificOptions): Promise<TransformResult<SpecificOptions>>;
 export declare function transform(input: string | Uint8Array, options?: TransformOptions): Promise<TransformResult>;
 
 /**
@@ -540,7 +541,7 @@ export declare function analyzeMetafile(metafile: Metafile | string, options?: A
  *
  * Documentation: https://esbuild.github.io/api/#build-api
  */
-export declare function buildSync(options: BuildOptions & { write: false }): BuildResult & { outputFiles: OutputFile[] };
+export declare function buildSync<SpecificOptions extends BuildOptions>(options: SpecificOptions): BuildResult<SpecificOptions>;
 export declare function buildSync(options: BuildOptions): BuildResult;
 
 /**
@@ -551,7 +552,8 @@ export declare function buildSync(options: BuildOptions): BuildResult;
  *
  * Documentation: https://esbuild.github.io/api/#transform-api
  */
-export declare function transformSync(input: string, options?: TransformOptions): TransformResult;
+export declare function transformSync<SpecificOptions extends TransformOptions>(input: string, options?: SpecificOptions): TransformResult<SpecificOptions>;
+export declare function transformSync(input: string | Uint8Array, options?: TransformOptions): TransformResult;
 
 /**
  * A synchronous version of "formatMessages".
