@@ -3,7 +3,7 @@
 //
 // If you are just trying to run esbuild from Go without the overhead of
 // creating a child process, there is also an API for the command-line
-// interface itself: https://godoc.org/github.com/evanw/esbuild/pkg/cli.
+// interface itself: https://pkg.go.dev/github.com/evanw/esbuild/pkg/cli.
 //
 // # Build API
 //
@@ -76,6 +76,12 @@
 //	}
 package api
 
+import (
+	"time"
+
+	"github.com/evanw/esbuild/internal/logger"
+)
+
 type SourceMap uint8
 
 const (
@@ -104,12 +110,12 @@ const (
 	LegalCommentsExternal
 )
 
-type JSXMode uint8
+type JSX uint8
 
 const (
-	JSXModeTransform JSXMode = iota
-	JSXModePreserve
-	JSXModeAutomatic
+	JSXTransform JSX = iota
+	JSXPreserve
+	JSXAutomatic
 )
 
 type Target uint8
@@ -138,6 +144,7 @@ const (
 	LoaderCSS
 	LoaderDataURL
 	LoaderDefault
+	LoaderEmpty
 	LoaderFile
 	LoaderJS
 	LoaderJSON
@@ -165,6 +172,13 @@ const (
 	FormatESModule
 )
 
+type Packages uint8
+
+const (
+	PackagesDefault Packages = iota
+	PackagesExternal
+)
+
 type Engine struct {
 	Name    EngineName
 	Version string
@@ -276,12 +290,12 @@ type BuildOptions struct {
 	IgnoreAnnotations bool                   // Documentation: https://esbuild.github.io/api/#ignore-annotations
 	LegalComments     LegalComments          // Documentation: https://esbuild.github.io/api/#legal-comments
 
-	JSXMode         JSXMode // Documentation: https://esbuild.github.io/api/#jsx-mode
-	JSXFactory      string  // Documentation: https://esbuild.github.io/api/#jsx-factory
-	JSXFragment     string  // Documentation: https://esbuild.github.io/api/#jsx-fragment
-	JSXImportSource string  // Documentation: https://esbuild.github.io/api/#jsx-import-source
-	JSXDev          bool    // Documentation: https://esbuild.github.io/api/#jsx-dev
-	JSXSideEffects  bool    // Documentation: https://esbuild.github.io/api/#jsx-side-effects
+	JSX             JSX    // Documentation: https://esbuild.github.io/api/#jsx-mode
+	JSXFactory      string // Documentation: https://esbuild.github.io/api/#jsx-factory
+	JSXFragment     string // Documentation: https://esbuild.github.io/api/#jsx-fragment
+	JSXImportSource string // Documentation: https://esbuild.github.io/api/#jsx-import-source
+	JSXDev          bool   // Documentation: https://esbuild.github.io/api/#jsx-dev
+	JSXSideEffects  bool   // Documentation: https://esbuild.github.io/api/#jsx-side-effects
 
 	Define    map[string]string // Documentation: https://esbuild.github.io/api/#define
 	Pure      []string          // Documentation: https://esbuild.github.io/api/#pure
@@ -299,13 +313,14 @@ type BuildOptions struct {
 	Platform          Platform          // Documentation: https://esbuild.github.io/api/#platform
 	Format            Format            // Documentation: https://esbuild.github.io/api/#format
 	External          []string          // Documentation: https://esbuild.github.io/api/#external
+	Packages          Packages          // Documentation: https://esbuild.github.io/api/#packages
 	Alias             map[string]string // Documentation: https://esbuild.github.io/api/#alias
 	MainFields        []string          // Documentation: https://esbuild.github.io/api/#main-fields
 	Conditions        []string          // Documentation: https://esbuild.github.io/api/#conditions
 	Loader            map[string]Loader // Documentation: https://esbuild.github.io/api/#loader
 	ResolveExtensions []string          // Documentation: https://esbuild.github.io/api/#resolve-extensions
 	Tsconfig          string            // Documentation: https://esbuild.github.io/api/#tsconfig
-	OutExtensions     map[string]string // Documentation: https://esbuild.github.io/api/#out-extension
+	OutExtension      map[string]string // Documentation: https://esbuild.github.io/api/#out-extension
 	PublicPath        string            // Documentation: https://esbuild.github.io/api/#public-path
 	Inject            []string          // Documentation: https://esbuild.github.io/api/#inject
 	Banner            map[string]string // Documentation: https://esbuild.github.io/api/#banner
@@ -322,10 +337,7 @@ type BuildOptions struct {
 	Stdin          *StdinOptions // Documentation: https://esbuild.github.io/api/#stdin
 	Write          bool          // Documentation: https://esbuild.github.io/api/#write
 	AllowOverwrite bool          // Documentation: https://esbuild.github.io/api/#allow-overwrite
-	Incremental    bool          // Documentation: https://esbuild.github.io/api/#incremental
 	Plugins        []Plugin      // Documentation: https://esbuild.github.io/plugins/
-
-	Watch *WatchMode // Documentation: https://esbuild.github.io/api/#watch
 }
 
 type EntryPoint struct {
@@ -333,10 +345,6 @@ type EntryPoint struct {
 	OutputPath string
 }
 
-type WatchMode struct {
-	OnRebuild func(BuildResult)
-}
-
 type StdinOptions struct {
 	Contents   string
 	ResolveDir string
@@ -351,9 +359,6 @@ type BuildResult struct {
 	OutputFiles []OutputFile
 	Metafile    string
 	MangleCache map[string]interface{}
-
-	Rebuild func() BuildResult // Only when "Incremental: true"
-	Stop    func()             // Only when "Watch: true"
 }
 
 type OutputFile struct {
@@ -363,7 +368,23 @@ type OutputFile struct {
 
 // Documentation: https://esbuild.github.io/api/#build-api
 func Build(options BuildOptions) BuildResult {
-	return buildImpl(options).result
+	start := time.Now()
+
+	ctx, errors := contextImpl(options)
+	if ctx == nil {
+		return BuildResult{Errors: errors}
+	}
+
+	result := ctx.Rebuild()
+
+	// Print a summary of the generated files to stderr. Except don't do
+	// this if the terminal is already being used for something else.
+	if ctx.args.logOptions.LogLevel <= logger.LevelInfo && !ctx.args.options.WriteToStdout {
+		printSummary(ctx.args.logOptions.Color, result.OutputFiles, start)
+	}
+
+	ctx.Dispose()
+	return result
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -400,12 +421,12 @@ type TransformOptions struct {
 	IgnoreAnnotations bool                   // Documentation: https://esbuild.github.io/api/#ignore-annotations
 	LegalComments     LegalComments          // Documentation: https://esbuild.github.io/api/#legal-comments
 
-	JSXMode         JSXMode // Documentation: https://esbuild.github.io/api/#jsx
-	JSXFactory      string  // Documentation: https://esbuild.github.io/api/#jsx-factory
-	JSXFragment     string  // Documentation: https://esbuild.github.io/api/#jsx-fragment
-	JSXImportSource string  // Documentation: https://esbuild.github.io/api/#jsx-import-source
-	JSXDev          bool    // Documentation: https://esbuild.github.io/api/#jsx-dev
-	JSXSideEffects  bool    // Documentation: https://esbuild.github.io/api/#jsx-side-effects
+	JSX             JSX    // Documentation: https://esbuild.github.io/api/#jsx
+	JSXFactory      string // Documentation: https://esbuild.github.io/api/#jsx-factory
+	JSXFragment     string // Documentation: https://esbuild.github.io/api/#jsx-fragment
+	JSXImportSource string // Documentation: https://esbuild.github.io/api/#jsx-import-source
+	JSXDev          bool   // Documentation: https://esbuild.github.io/api/#jsx-dev
+	JSXSideEffects  bool   // Documentation: https://esbuild.github.io/api/#jsx-side-effects
 
 	TsconfigRaw string // Documentation: https://esbuild.github.io/api/#tsconfig-raw
 	Banner      string // Documentation: https://esbuild.github.io/api/#banner
@@ -423,8 +444,9 @@ type TransformResult struct {
 	Errors   []Message
 	Warnings []Message
 
-	Code []byte
-	Map  []byte
+	Code          []byte
+	Map           []byte
+	LegalComments []byte
 
 	MangleCache map[string]interface{}
 }
@@ -435,13 +457,15 @@ func Transform(input string, options TransformOptions) TransformResult {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Serve API
+// Context API
 
 // Documentation: https://esbuild.github.io/api/#serve-arguments
 type ServeOptions struct {
 	Port      uint16
 	Host      string
 	Servedir  string
+	Keyfile   string
+	Certfile  string
 	OnRequest func(ServeOnRequestArgs)
 }
 
@@ -457,13 +481,39 @@ type ServeOnRequestArgs struct {
 type ServeResult struct {
 	Port uint16
 	Host string
-	Wait func() error
-	Stop func()
 }
 
-// Documentation: https://esbuild.github.io/api/#serve
-func Serve(serveOptions ServeOptions, buildOptions BuildOptions) (ServeResult, error) {
-	return serveImpl(serveOptions, buildOptions)
+type OnChangeArgs struct {
+}
+
+type WatchOptions struct {
+}
+
+type BuildContext interface {
+	Rebuild() BuildResult
+	Watch(options WatchOptions) error
+	Serve(options ServeOptions) (ServeResult, error)
+	Dispose()
+}
+
+type ContextError struct {
+	Errors []Message // Option validation errors are returned here
+}
+
+func (err *ContextError) Error() string {
+	if len(err.Errors) > 0 {
+		return err.Errors[0].Text
+	}
+	return "Context creation failed"
+}
+
+// Documentation: https://esbuild.github.io/api/#context-api
+func Context(buildOptions BuildOptions) (BuildContext, *ContextError) {
+	ctx, errors := contextImpl(buildOptions)
+	if ctx == nil {
+		return nil, &ContextError{Errors: errors}
+	}
+	return ctx, nil
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -485,7 +535,7 @@ type PluginBuild struct {
 	InitialOptions *BuildOptions
 	Resolve        func(path string, options ResolveOptions) ResolveResult
 	OnStart        func(callback func() (OnStartResult, error))
-	OnEnd          func(callback func(result *BuildResult))
+	OnEnd          func(callback func(result *BuildResult) (OnEndResult, error))
 	OnResolve      func(options OnResolveOptions, callback func(OnResolveArgs) (OnResolveResult, error))
 	OnLoad         func(options OnLoadOptions, callback func(OnLoadArgs) (OnLoadResult, error))
 }
@@ -516,6 +566,11 @@ type OnStartResult struct {
 	Warnings []Message
 }
 
+type OnEndResult struct {
+	Errors   []Message
+	Warnings []Message
+}
+
 type OnResolveOptions struct {
 	Filter    string
 	Namespace string
@@ -577,7 +632,8 @@ type OnLoadResult struct {
 type ResolveKind uint8
 
 const (
-	ResolveEntryPoint ResolveKind = iota
+	ResolveNone ResolveKind = iota
+	ResolveEntryPoint
 	ResolveJSImportStatement
 	ResolveJSRequireCall
 	ResolveJSDynamicImport
