@@ -7,6 +7,15 @@ import "fmt"
 
 // Remove the serve API in the WebAssembly build. This removes 2.7mb of stuff.
 
-func serveImpl(serveOptions ServeOptions, buildOptions BuildOptions) (ServeResult, error) {
+func (*internalContext) Serve(ServeOptions) (ServeResult, error) {
 	return ServeResult{}, fmt.Errorf("The \"serve\" API is not supported when using WebAssembly")
 }
+
+type apiHandler struct {
+}
+
+func (*apiHandler) broadcastBuildResult(BuildResult, buildSummary) {
+}
+
+func (*apiHandler) stop() {
+}
